from dataclasses import dataclass, field

from typing import Any, Callable, List, Optional
import numpy as np

from leapcli.dataset.contract import DatasetInputType, DatasetOutputType, DatasetMetadataType


@dataclass
class SubsetResponse:
    length: int
    data: Any


SectionCallableInterface = Callable[[int, SubsetResponse], np.ndarray]


@dataclass
class SubsetHandler:
    ratio: float
    function: Callable[[], List[SubsetResponse]]
    name: str


@dataclass
class DatasetBaseHandler:
    name: str
    function: SectionCallableInterface
    subset: str


@dataclass
class InputHandler(DatasetBaseHandler):
    type: DatasetInputType
    shape: Optional[List[int]]


@dataclass
class GroundTruthHandler(DatasetBaseHandler):
    type: DatasetOutputType
    labels: List[str]
    masked_input: Optional[str]
    shape: Optional[List[int]]


@dataclass
class MetadataHandler(DatasetBaseHandler):
    type: DatasetMetadataType


@dataclass
class  DatasetIntegrationSetup:
    subsets: List[SubsetHandler] = field(default_factory=list)
    inputs: List[InputHandler] = field(default_factory=list)
    ground_truths: List[GroundTruthHandler] = field(default_factory=list)
    metadata: List[MetadataHandler] = field(default_factory=list)


class DatasetBinder:

    def __init__(self):
        self._container = DatasetIntegrationSetup()

    def set_subset(self, ratio: float, function: Callable[[None], List[SubsetResponse]], name: str) -> None:
        self._container.subsets.append(SubsetHandler(ratio, function, name))

    def set_input(self, function: SectionCallableInterface, subset: str,
                  input_type: DatasetInputType, name: str) -> None:
        self._container.inputs.append(InputHandler(name, function, subset, input_type, []))

    def set_ground_truth(self, function: SectionCallableInterface, subset: str,
                         ground_truth_type: DatasetOutputType, name: str, labels: List[str],
                         masked_input: Optional[str]) -> None:
        self._container.ground_truths.append(GroundTruthHandler(name, function, subset,
                                                                ground_truth_type, labels, masked_input, []))

    def set_metadata(self, function: SectionCallableInterface, subset: str,
                     metadata_type: DatasetMetadataType, name: str) -> None:
        self._container.metadata.append(MetadataHandler(name, function, subset, metadata_type))



from enum import Enum
from typing import List, Dict, Optional

from dataclasses import dataclass


class DatasetOutputType(Enum):
    Numeric = "Numeric"
    Classes = "Classes"
    Mask = "Mask"


class DatasetInputType(Enum):
    Image = "Image"
    Text = "Text"
    Word = "Word"
    Numeric = "Numeric"
    Time_series = "Time_series"


class DatasetParameterType(Enum):
    Number = "Number"
    Select = "Select"
    String = "String"
    MultiSelect = "MultiSelect"


class DatasetMetadataType(Enum):
    float = "float"
    string = "string"
    int = "int"
    boolean = "boolean"


@dataclass
class DatasetSubsetInstance:
    id: str
    name: str
    ratio: float
    script: str


@dataclass
class DatasetBaseSectionInstance:
    id: str
    name: str
    script: str
    subset: str


@dataclass
class DatasetInputInstance(DatasetBaseSectionInstance):
    shape: List[int]
    type: DatasetInputType


@dataclass
class DatasetLibInstance:
    as_: str
    from_: str
    id: str
    import_: str


@dataclass
class DatasetMetadataInstance(DatasetBaseSectionInstance):
    type: DatasetMetadataType


@dataclass
class DatasetOutputInstance(DatasetBaseSectionInstance):
    shape: List[int]
    type: DatasetOutputType
    masked_input: Optional[str] = None
    labels: Optional[List[str]] = None


@dataclass
class DatasetParameterInstance:
    id: str
    name: str
    default_value: str
    type: DatasetParameterType


@dataclass
class DatasetSetup:
    inputs: List[DatasetInputInstance]
    libs: List[DatasetLibInstance]
    metadata: List[DatasetMetadataInstance]
    outputs: List[DatasetOutputInstance]
    parameters: List[DatasetParameterInstance]
    subsets: List[DatasetSubsetInstance]


class CloudProviderEnum(Enum):
    aws = "aws"
    gcloud = "gcloud"


@dataclass
class Secret:
    name: str
    cloud_provider: CloudProviderEnum
    auth_file_path: str


@dataclass
class BucketConfig:
    bucket: str
    secret: Secret


@dataclass
class DatasetRequest:
    name: str
    setup: DatasetSetup
    input_shape: Dict[str, List[int]]
    bucket_config: Optional[BucketConfig] = None
